@@ -25,9 +25,13 @@ class MemberEvents(EventMixinTemplate):
     async def _on_raw_guild_member_remove(self, event: "RawGatewayEvent") -> None:
         g_id = event.data.pop("guild_id")
         user = self.cache.place_user_data(event.data["user"])
+        member = self.cache.get_member(g_id, user.id)
+
+        self.cache.delete_member(g_id, user.id)
         guild = self.cache.get_guild(g_id)
         guild.member_count -= 1
-        self.dispatch(events.MemberRemove(g_id, self.cache.get_member(g_id, user.id) or user))
+
+        self.dispatch(events.MemberRemove(g_id, member or user))
 
     @Processor.define()
     async def _on_raw_guild_member_update(self, event: "RawGatewayEvent") -> None:
