@@ -1,10 +1,8 @@
 import copy
-import logging
 from typing import TYPE_CHECKING
 
 import naff.api.events as events
 
-from naff.client.const import logger_name
 from ._template import EventMixinTemplate, Processor
 
 if TYPE_CHECKING:
@@ -12,8 +10,6 @@ if TYPE_CHECKING:
 
 __all__ = ("VoiceEvents",)
 
-log = logging.getLogger(logger_name)
-
 
 class VoiceEvents(EventMixinTemplate):
     @Processor.define()
