@@ -3,7 +3,7 @@ import inspect
 import typing
 from collections import deque
 from types import NoneType, UnionType
-from typing import Optional, Any, Callable, Annotated, Literal, Union, TYPE_CHECKING, Type
+from typing import Optional, Any, Callable, Annotated, Literal, Union, TYPE_CHECKING, Type, TypeGuard
 
 import attrs
 
@@ -110,6 +110,10 @@ class _PrefixedArgsIterator:
         return self.index >= self.length
 
 
+def _check_for_no_arg(anno: Any) -> TypeGuard[NoArgumentConverter]:
+    return isinstance(anno, NoArgumentConverter) or (inspect.isclass(anno) and issubclass(anno, NoArgumentConverter))
+
+
 def _convert_to_bool(argument: str) -> bool:
     lowered = argument.lower()
     if lowered in {"yes", "y", "true", "t", "1", "enable", "on"}:
@@ -144,13 +148,17 @@ def _get_converter(anno: type, name: str) -> Callable[["PrefixedContext", str],
     elif typing.get_origin(anno) is Literal:
         literals = typing.get_args(anno)
         return _LiteralConverter(literals).convert
-    elif inspect.isfunction(anno):
+    elif inspect.isroutine(anno):
         num_params = len(inspect.signature(anno).parameters.values())
         match num_params:
             case 2:
-                return lambda ctx, arg: anno(ctx, arg)
+                return anno
             case 1:
-                return lambda ctx, arg: anno(arg)
+
+                async def _one_function_cmd(ctx, arg) -> Any:
+                    return await maybe_coroutine(anno, arg)
+
+                return _one_function_cmd
             case 0:
                 ValueError(f"{get_object_name(anno)} for {name} has 0 arguments, which is unsupported.")
             case _:
@@ -433,7 +441,7 @@ class PrefixedCommand(BaseCommand):
             if typing.get_origin(anno) in {Union, UnionType}:
                 cmd_param.union = True
                 for arg in typing.get_args(anno):
-                    if isinstance(anno, NoArgumentConverter):
+                    if _check_for_no_arg(anno):
                         cmd_param.no_argument = True
 
                     if arg != NoneType:
@@ -442,7 +450,7 @@ class PrefixedCommand(BaseCommand):
                     elif not cmd_param.optional:  # d.py-like behavior
                         cmd_param.default = None
             else:
-                if isinstance(anno, NoArgumentConverter):
+                if _check_for_no_arg(anno):
                     cmd_param.no_argument = True
 
                 converter = _get_converter(anno, name)
@@ -480,17 +488,16 @@ class PrefixedCommand(BaseCommand):
         """
         cmd.parent = self  # just so we know this is a subcommand
 
-        cmd_names = frozenset(self.subcommands)
-        if cmd.name in cmd_names:
+        if self.subcommands.get(cmd.name):
             raise ValueError(
-                f"Duplicate Command! Multiple commands share the name/alias `{self.qualified_name} {cmd.name}`"
+                f"Duplicate command! Multiple commands share the name/alias: {self.qualified_name} {cmd.name}."
             )
         self.subcommands[cmd.name] = cmd
 
         for alias in cmd.aliases:
-            if alias in cmd_names:
+            if self.subcommands.get(alias):
                 raise ValueError(
-                    f"Duplicate Command! Multiple commands share the name/alias `{self.qualified_name} {cmd.name}`"
+                    f"Duplicate command! Multiple commands share the name/alias: {self.qualified_name} {cmd.name}."
                 )
             self.subcommands[alias] = cmd
 
@@ -505,7 +512,11 @@ class PrefixedCommand(BaseCommand):
         """
         command = self.subcommands.pop(name, None)
 
-        if command is None or name in command.aliases:
+        if command is None:
+            return
+
+        if name in command.aliases:
+            command.aliases.remove(name)
             return
 
         for alias in command.aliases:
@@ -596,6 +607,8 @@ class PrefixedCommand(BaseCommand):
         """
         # sourcery skip: remove-empty-nested-block, remove-redundant-if, remove-unnecessary-else
         if len(self.parameters) == 0:
+            if ctx.args and not self.ignore_extra:
+                raise BadArgument(f"Too many arguments passed to {self.name}.")
             return await self.call_with_binding(callback, ctx)
         else:
             # this is slightly costly, but probably worth it
@@ -644,6 +657,15 @@ class PrefixedCommand(BaseCommand):
 
             if param_index < len(self.parameters):
                 for param in self.parameters[param_index:]:
+                    if param.no_argument:
+                        converted, _ = await _convert(param, ctx, None)  # type: ignore
+                        if not param.consume_rest:
+                            new_args.append(converted)
+                        else:
+                            kwargs[param.name] = converted
+                            break
+                        continue
+
                     if not param.optional:
                         raise BadArgument(f"{param.name} is a required argument that is missing.")
                     else:
